@@ -17,8 +17,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-from __future__ import absolute_import
-
 import email.utils
 import io
 import os
@@ -28,21 +26,16 @@ import re
 import subprocess
 import sys
 import tempfile
-if sys.version_info[0] >= 3:
-    import unittest
-else:
-    import unittest2 as unittest
+import textwrap
+import unittest
 import warnings
 
-import six
-
-if six.PY3:
-    from io import BytesIO, StringIO
-else:
-    from StringIO import StringIO
-    BytesIO = StringIO
-
-import apt_pkg
+try:
+    # skip tests that require apt_pkg if it is not available
+    import apt_pkg
+    _have_apt_pkg = True
+except (ImportError, AttributeError):
+    _have_apt_pkg = False
 
 from debian import deb822
 from debian.debian_support import Version
@@ -181,7 +174,7 @@ CcYAoOLYDF5G1h3oR1iDNyeCI6hRW03S
     ]
 
 
-CHANGES_FILE = six.u('''\
+CHANGES_FILE = '''\
 Format: 1.7
 Date: Fri, 28 Dec 2007 17:08:48 +0100
 Source: bzr-gtk
@@ -206,7 +199,7 @@ Files:
  0fd797f4138a9d4fdeb8c30597d46bc9 1003 python optional bzr-gtk_0.93.0-2.dsc
  d9523676ae75c4ced299689456f252f4 3860 python optional bzr-gtk_0.93.0-2.diff.gz
  8960459940314b21019dedd5519b47a5 168544 python optional bzr-gtk_0.93.0-2_all.deb
-''')
+'''
 
 CHECKSUM_CHANGES_FILE = '''\
 Format: 1.8
@@ -334,16 +327,7 @@ KEYRING = os.path.abspath(find_test_file('test-keyring.gpg'))
 def open_utf8(filename, mode='r'):
     # type: (str, str) -> IO[Text]
     """Open a UTF-8 text file in text mode."""
-    if sys.version < '3':
-        # TODO(jsw): This isn't actually doing what the docstring says.  The
-        # correct code (for both 2 and 3) is
-        #   io.open(filename, code=mode, encoding='utf-8')
-        # but that makes a couple of other tests fail on 2.x (both related to
-        # apt_pkg - not surprisingly, its behavior with unicode objects isn't
-        # very consistent).
-        return open(filename, mode=mode)
-    else:
-        return open(filename, mode=mode, encoding='UTF-8')
+    return open(filename, mode=mode, encoding='UTF-8')
 
 
 class TestDeb822Dict(unittest.TestCase):
@@ -388,7 +372,6 @@ class TestDeb822Dict(unittest.TestCase):
         keys = ['TestKey', 'another_key', 'Third_key']
 
         self.assertEqual(keys, list(d.keys()))
-        self.assertEqual(keys, list(six.iterkeys(d)))
         self.assertEqual(list(zip(keys, d.values())), list(d.items()))
 
         keys2 = []
@@ -407,7 +390,7 @@ class TestDeb822Dict(unittest.TestCase):
     def test_unicode_key_access(self):
         # type: () -> None
         d = self.make_dict()
-        self.assertEqual(1, d[six.u('testkey')])
+        self.assertEqual(1, d['testkey'])
 
 
 class TestDeb822(unittest.TestCase):
@@ -481,6 +464,96 @@ with open("test_deb822.pickle", "wb") as fh:
         deb822_ = deb822.Deb822(['\n'] * 3 + UNPARSED_PACKAGE.splitlines())
         self.assertWellParsed(deb822_, PARSED_PACKAGE)
 
+    def test_reorder(self):
+        # type: () -> None
+        content = textwrap.dedent("""
+        Depends: bar
+        Description: some-text
+        Architecture: any
+        Package: foo
+        Recommends: baz
+        """)
+        paragraph = deb822.Deb822(content.splitlines())
+        # Verify the starting state
+        self.assertEqual(list(paragraph.keys()),
+                         ['Depends', 'Description', 'Architecture', 'Package', 'Recommends']
+                         )
+        # no op
+        paragraph.order_last('Recommends')
+        self.assertEqual(list(paragraph.keys()),
+                         ['Depends', 'Description', 'Architecture', 'Package', 'Recommends']
+                         )
+        # no op
+        paragraph.order_first('Depends')
+        self.assertEqual(list(paragraph.keys()),
+                         ['Depends', 'Description', 'Architecture', 'Package', 'Recommends']
+                         )
+
+        paragraph.order_first('Package')
+        self.assertEqual(list(paragraph.keys()),
+                         ['Package', 'Depends', 'Description', 'Architecture', 'Recommends']
+                         )
+
+        paragraph.order_last('Description')
+        self.assertEqual(list(paragraph.keys()),
+                         ['Package', 'Depends', 'Architecture', 'Recommends', 'Description']
+                         )
+
+        paragraph.order_after('Recommends', 'Depends')
+        self.assertEqual(list(paragraph.keys()),
+                         ['Package', 'Depends', 'Recommends', 'Architecture', 'Description']
+                         )
+
+        paragraph.order_before('Architecture', 'Depends')
+        self.assertEqual(list(paragraph.keys()),
+                         ['Package', 'Architecture', 'Depends', 'Recommends', 'Description']
+                         )
+
+        with self.assertRaises(ValueError):
+            paragraph.order_after('Architecture', 'Architecture')
+        with self.assertRaises(ValueError):
+            paragraph.order_before('Architecture', 'Architecture')
+        with self.assertRaises(KeyError):
+            paragraph.order_before('Unknown-Field', 'Architecture')
+        with self.assertRaises(KeyError):
+            paragraph.order_before('Architecture', 'Unknown-Field')
+
+    def test_sort_fields(self):
+        # type: () -> None
+        content = textwrap.dedent("""
+        Depends: bar
+        Description: some-text
+        Architecture: any
+        Package: foo
+        Recommends: baz
+        """)
+        paragraph = deb822.Deb822(content.splitlines())
+        # Initial state
+        self.assertEqual(list(paragraph.keys()),
+                         ['Depends', 'Description', 'Architecture', 'Package', 'Recommends']
+                         )
+
+        # Sorting defaults to using name
+        paragraph.sort_fields()
+        self.assertEqual(list(paragraph.keys()),
+                         ['Architecture', 'Depends', 'Description', 'Package', 'Recommends']
+                         )
+
+        # Sorting using a key function
+        order_table = {
+            'package': 1,
+            'architecture': 2,
+            # Dependencies have the same order
+            'depends': 3,
+            'recommends': 3,
+            # Desc definitely last
+            'description': 100,
+        }
+        paragraph.sort_fields(key=lambda x: order_table[x.lower()])
+        self.assertEqual(list(paragraph.keys()),
+                         ['Package', 'Architecture', 'Depends', 'Recommends', 'Description']
+                         )
+
     def test_gpg_stripping(self):
         # type: () -> None
         for string in GPG_SIGNED:
@@ -494,7 +567,7 @@ with open("test_deb822.pickle", "wb") as fh:
         unparsed_with_gpg = SIGNED_CHECKSUM_CHANGES_FILE % CHECKSUM_CHANGES_FILE
         deb822_from_str = deb822.Dsc(unparsed_with_gpg)
         result_from_str = deb822_from_str.get_gpg_info(keyrings=[KEYRING])
-        deb822_from_file = deb822.Dsc(StringIO(unparsed_with_gpg))
+        deb822_from_file = deb822.Dsc(io.StringIO(unparsed_with_gpg))
         result_from_file = deb822_from_file.get_gpg_info(keyrings=[KEYRING])
         deb822_from_lines = deb822.Dsc(unparsed_with_gpg.splitlines())
         result_from_lines = deb822_from_lines.get_gpg_info(keyrings=[KEYRING])
@@ -536,39 +609,52 @@ with open("test_deb822.pickle", "wb") as fh:
 
     def test_iter_paragraphs_file_io(self):
         # type: () -> None
-        text = StringIO(UNPARSED_PACKAGE + '\n\n\n' + UNPARSED_PACKAGE)
+        text = io.StringIO(UNPARSED_PACKAGE + '\n\n\n' + UNPARSED_PACKAGE)
 
         for d in deb822.Deb822.iter_paragraphs(text, use_apt_pkg=False):
             self.assertWellParsed(d, PARSED_PACKAGE)
 
+    @unittest.skipUnless(_have_apt_pkg, "apt_pkg is not available")
+    def test_iter_paragraphs_file_io_apt_pkg(self):
+        # type: () -> None
+        text = io.StringIO(UNPARSED_PACKAGE + '\n\n\n' + UNPARSED_PACKAGE)
+
         with self.assertWarns(UserWarning):
-            # The StringIO is not a real file so this will raise a warning
+            # The io.StringIO is not a real file so this will raise a warning
             for d in deb822.Deb822.iter_paragraphs(text, use_apt_pkg=True):
                 self.assertWellParsed(d, PARSED_PACKAGE)
 
     def test_iter_paragraphs_file(self):
         # type: () -> None
-        text = StringIO()
+        text = io.StringIO()
         text.write(UNPARSED_PACKAGE)
         text.write('\n\n\n')
         text.write(UNPARSED_PACKAGE)
 
         with tempfile.NamedTemporaryFile() as fh:
-            if sys.version_info[0] >= 3:
-                txt = text.getvalue().encode('UTF-8')
-            else:
-                txt = text.getvalue()
+            txt = text.getvalue().encode('UTF-8')
             fh.write(txt)
 
             fh.seek(0)
             for d in deb822.Deb822.iter_paragraphs(fh, use_apt_pkg=False):
                 self.assertWellParsed(d, PARSED_PACKAGE)
 
+    @unittest.skipUnless(_have_apt_pkg, "apt_pkg is not available")
+    def test_iter_paragraphs_file_apt_pkg(self):
+        # type: () -> None
+        text = io.StringIO()
+        text.write(UNPARSED_PACKAGE)
+        text.write('\n\n\n')
+        text.write(UNPARSED_PACKAGE)
+
+        with tempfile.NamedTemporaryFile() as fh:
+            txt = text.getvalue().encode('UTF-8')
+            fh.write(txt)
+
             fh.seek(0)
             for d in deb822.Deb822.iter_paragraphs(fh, use_apt_pkg=True):
                 self.assertWellParsed(d, PARSED_PACKAGE)
 
-
     def test_iter_paragraphs_with_gpg(self):
         # type: () -> None
         for string in GPG_SIGNED:
@@ -585,16 +671,26 @@ with open("test_deb822.pickle", "wb") as fh:
     def test_iter_paragraphs_bytes(self):
         # type: () -> None
         text = (UNPARSED_PACKAGE + '\n\n\n' + UNPARSED_PACKAGE)
-        if six.PY2:
-            binary = text
-        else:
-            binary = text.encode('utf-8')
+        binary = text.encode('utf-8')
 
         for d in deb822.Deb822.iter_paragraphs(binary):
             self.assertWellParsed(d, PARSED_PACKAGE)
 
-    def test_iter_paragraphs_with_extra_whitespace(self):
-        # type: () -> None
+    def _test_iter_paragraphs_count(self, filename, cmd, expected, *args, **kwargs):
+        # type: (str, Callable[..., Any], int, *Any, **Any) -> None
+        with open_utf8(filename) as fh:
+            count = len(list(cmd(fh, *args, **kwargs)))
+            self.assertEqual(
+                expected,
+                count,
+                "Wrong number paragraphs were found: expected {expected}, got {count}".format(
+                    count=count,
+                    expected=expected,
+                )
+            )
+
+    def _test_iter_paragraphs_with_extra_whitespace(self, tests):
+        # type: (Callable[[str], None]) -> None
         """ Paragraphs splitting when stray whitespace is between
 
         From policy §5.1:
@@ -613,55 +709,66 @@ with open("test_deb822.pickle", "wb") as fh:
         See #913274 for further details.
         """
         for extra_space in (" ", "  ", "\t"):
-            text = six.u(UNPARSED_PACKAGE) + '%s\n' % extra_space + \
-                        six.u(UNPARSED_PACKAGE)
+            text = UNPARSED_PACKAGE + '%s\n' % extra_space + \
+                        UNPARSED_PACKAGE
 
             fd, filename = tempfile.mkstemp()
             fp = os.fdopen(fd, 'wb')
             fp.write(text.encode('utf-8'))
             fp.close()
 
-            def test_count(cmd, expected, *args, **kwargs):
-                # type: (Callable[..., Any], int, *Any, **Any) -> None
-                #print("\n", cmd.__class__, expected)
-                with open_utf8(filename) as fh:
-                    count = len(list(cmd(fh, *args, **kwargs)))
-                    self.assertEqual(
-                        expected,
-                        count,
-                        "Wrong number paragraphs were found: expected {expected}, got {count}".format(
-                            count=count,
-                            expected=expected,
-                        )
-                    )
-
             try:
-                # apt_pkg not used, should split
-                test_count(deb822.Deb822.iter_paragraphs, 2)
-                test_count(deb822.Deb822.iter_paragraphs, 2, use_apt_pkg=False)
-
-                # apt_pkg used, should not split
-                test_count(deb822.Deb822.iter_paragraphs, 1, use_apt_pkg=True)
-
-                # Specialised iter_paragraphs force use of apt_pkg and don't split
-                test_count(deb822.Packages.iter_paragraphs, 1, use_apt_pkg=True)
-                test_count(deb822.Sources.iter_paragraphs, 1, use_apt_pkg=True)
-                test_count(deb822.Packages.iter_paragraphs, 1, use_apt_pkg=False)
-                test_count(deb822.Sources.iter_paragraphs, 1, use_apt_pkg=False)
-
-                # Explicitly set internal parser to not split
-                strict = {'whitespace-separates-paragraphs': False}
-                test_count(deb822.Packages.iter_paragraphs, 1, use_apt_pkg=False, strict=strict)
-                test_count(deb822.Sources.iter_paragraphs, 1, use_apt_pkg=False, strict=strict)
-
-                # Explicitly set internal parser to split
-                strict = {'whitespace-separates-paragraphs': True}
-                test_count(deb822.Packages.iter_paragraphs, 2, use_apt_pkg=False, strict=strict)
-                test_count(deb822.Sources.iter_paragraphs, 2, use_apt_pkg=False, strict=strict)
-
+                tests(filename)
             finally:
                 os.remove(filename)
 
+    def test_iter_paragraphs_with_extra_whitespace_default(self):
+        # type: () -> None
+        """ Paragraphs splitting with stray whitespace (default options) """
+        def tests(filename): # type: (str) -> None
+            # apt_pkg not used, should split
+            self._test_iter_paragraphs_count(filename, deb822.Deb822.iter_paragraphs, 2)
+
+        self._test_iter_paragraphs_with_extra_whitespace(tests)
+
+    def test_iter_paragraphs_with_extra_whitespace_no_apt_pkg(self):
+        # type: () -> None
+        """ Paragraphs splitting with stray whitespace (without apt_pkg)"""
+        def tests(filename): # type: (str) -> None
+            # apt_pkg not used, should split
+            self._test_iter_paragraphs_count(filename, deb822.Deb822.iter_paragraphs, 2, use_apt_pkg=False)
+
+            # Specialised iter_paragraphs force use of apt_pkg and don't split
+            self._test_iter_paragraphs_count(filename, deb822.Packages.iter_paragraphs, 1, use_apt_pkg=False)
+            self._test_iter_paragraphs_count(filename, deb822.Sources.iter_paragraphs, 1, use_apt_pkg=False)
+
+            # Explicitly set internal parser to not split
+            strict = {'whitespace-separates-paragraphs': False}
+            self._test_iter_paragraphs_count(filename, deb822.Packages.iter_paragraphs, 1, use_apt_pkg=False, strict=strict)
+            self._test_iter_paragraphs_count(filename, deb822.Sources.iter_paragraphs, 1, use_apt_pkg=False, strict=strict)
+
+            # Explicitly set internal parser to split
+            strict = {'whitespace-separates-paragraphs': True}
+            self._test_iter_paragraphs_count(filename, deb822.Packages.iter_paragraphs, 2, use_apt_pkg=False, strict=strict)
+            self._test_iter_paragraphs_count(filename, deb822.Sources.iter_paragraphs, 2, use_apt_pkg=False, strict=strict)
+
+        self._test_iter_paragraphs_with_extra_whitespace(tests)
+
+    @unittest.skipUnless(_have_apt_pkg, "apt_pkg is not available")
+    def test_iter_paragraphs_with_extra_whitespace_apt_pkg(self):
+        # type: () -> None
+        """ Paragraphs splitting with stray whitespace (with apt_pkg) """
+        def tests(filename): # type: (str) -> None
+
+            # apt_pkg used, should not split
+            self._test_iter_paragraphs_count(filename, deb822.Deb822.iter_paragraphs, 1, use_apt_pkg=True)
+
+            # Specialised iter_paragraphs force use of apt_pkg and don't split
+            self._test_iter_paragraphs_count(filename, deb822.Packages.iter_paragraphs, 1, use_apt_pkg=True)
+            self._test_iter_paragraphs_count(filename, deb822.Sources.iter_paragraphs, 1, use_apt_pkg=True)
+
+        self._test_iter_paragraphs_with_extra_whitespace(tests)
+
     def _test_iter_paragraphs(self, filename, cls, **kwargs):
         # type: (str, Type[deb822.Deb822], **Any) -> None
         """Ensure iter_paragraphs consistency"""
@@ -674,7 +781,7 @@ with open("test_deb822.pickle", "wb") as fh:
         packages_content = b"\n".join([line.rstrip() for line in
                                        packages_content.splitlines()] + [b''])
 
-        s = BytesIO()
+        s = io.BytesIO()
         l = []
         f = open_utf8(filename)
         for p in cls.iter_paragraphs(f, **kwargs):
@@ -686,7 +793,7 @@ with open("test_deb822.pickle", "wb") as fh:
         if kwargs["shared_storage"] is False:
             # If shared_storage is False, data should be consistent across
             # iterations -- i.e. we can use "old" objects
-            s = BytesIO()
+            s = io.BytesIO()
             for p in l:
                 p.dump(s)
                 s.write(b"\n")
@@ -818,7 +925,7 @@ with open("test_deb822.pickle", "wb") as fh:
         c_i_dict['TeSt-KeY'] = test_string_2
         self.assertEqual(test_string_2, c_i_dict['Test-Key'])
 
-        deb822_ = deb822.Deb822(StringIO(UNPARSED_PACKAGE))
+        deb822_ = deb822.Deb822(io.StringIO(UNPARSED_PACKAGE))
         # deb822_.keys() will return non-normalized keys
         for k in deb822_:
             self.assertEqual(deb822_[k], deb822_[k.lower()])
@@ -961,7 +1068,7 @@ Description: python modules to work with Debian-related data formats
                 fb, encoding="iso8859-1"))
         for d in objects:
             for value in d.values():
-                self.assertTrue(isinstance(value, six.text_type))
+                self.assertTrue(isinstance(value, str))
 
         # The same should be true for Sources and Changes except for their
         # _multivalued fields
@@ -974,7 +1081,7 @@ Description: python modules to work with Debian-related data formats
         for d in multi:
             for key, value in d.items():
                 if key.lower() not in d.__class__._multivalued_fields:
-                    self.assertTrue(isinstance(value, six.text_type))
+                    self.assertTrue(isinstance(value, str))
 
     def test_encoding_integrity(self):
         # type: () -> None
@@ -996,12 +1103,12 @@ Description: python modules to work with Debian-related data formats
         with open(find_test_file('test_Sources.iso8859-1'), 'rb') as fb:
             latin1_contents = b"\n".join([line.rstrip() for line in fb] + [b''])
 
-        utf8_to_latin1 = BytesIO()
+        utf8_to_latin1 = io.BytesIO()
         for d in utf8:
             d.dump(fd=utf8_to_latin1, encoding='iso8859-1')
             utf8_to_latin1.write(b"\n")
 
-        latin1_to_utf8 = BytesIO()
+        latin1_to_utf8 = io.BytesIO()
         for d in latin1:
             d.dump(fd=latin1_to_utf8, encoding='utf-8')
             latin1_to_utf8.write(b"\n")
@@ -1030,10 +1137,10 @@ Description: python modules to work with Debian-related data formats
                     ]:
                 p1 = next(paragraphs)
                 self.assertEqual(p1['maintainer'],
-                                six.u('Adeodato Sim\xf3 <dato@net.com.org.es>'))
+                                'Adeodato Sim\xf3 <dato@net.com.org.es>')
                 p2 = next(paragraphs)
                 self.assertEqual(p2['uploaders'],
-                                six.u('Frank K\xfcster <frank@debian.org>'))
+                                'Frank K\xfcster <frank@debian.org>')
 
     def test_dump_text_mode(self):
         # type: () -> None
@@ -1091,6 +1198,7 @@ Description: python modules to work with Debian-related data formats
             self.assertWellParsed(paragraphs[i],
                                   PARSED_PARAGRAPHS_WITH_COMMENTS[i])
 
+    @unittest.skipUnless(_have_apt_pkg, "apt_pkg is not available")
     def test_iter_paragraphs_comments_use_apt_pkg(self):
         # type: () -> None
         """ apt_pkg does not support comments within multiline fields
@@ -1385,8 +1493,12 @@ class TestPkgRelations(unittest.TestCase):
             self.assertEqual(src_rel,
                     deb822.PkgRelation.str(deb822.PkgRelation.parse_relations( \
                             src_rel)))
-        with self.assertWarns(UserWarning):
+        with self.assertLogs('debian.deb822', level='WARNING') as cm:
             deb822.PkgRelation.parse_relations("foo bar")
+            self.assertEqual(
+                cm.output,
+                ['WARNING:debian.deb822:cannot parse package relationship '
+                 '"foo bar", returning it raw'])
 
     def test_sources(self):
         # type: () -> None
@@ -1398,6 +1510,7 @@ class TestPkgRelations(unittest.TestCase):
         pkg1 = next(pkgs)
         rel1 = {'build-conflicts': [],
                 'build-conflicts-indep': [],
+                'build-conflicts-arch': [],
                 'build-depends': [
                         [rel({'name': 'apache2-src', 'version': ('>=', '2.2.9')})],
                         [rel({'name': 'libaprutil1-dev'})],
@@ -1407,6 +1520,7 @@ class TestPkgRelations(unittest.TestCase):
                         [rel({'name': 'debhelper', 'version': ('>>', '5.0.0')})],
                     ],
                 'build-depends-indep': [],
+                'build-depends-arch': [],
                 'binary': [
                         [rel({'name': 'apache2-mpm-itk'})]
                     ]
@@ -1415,6 +1529,7 @@ class TestPkgRelations(unittest.TestCase):
         pkg2 = next(pkgs)
         rel2 = {'build-conflicts': [],
                 'build-conflicts-indep': [],
+                'build-conflicts-arch': [],
                 'build-depends': [
                         [rel({'name': 'dpkg-dev', 'version': ('>=', '1.13.9')})],
                         [rel({'name': 'autoconf', 'version': ('>=', '2.13')})],
@@ -1438,6 +1553,7 @@ class TestPkgRelations(unittest.TestCase):
                         [rel({'name': 'lsb-release'})],
                     ],
                 'build-depends-indep': [],
+                'build-depends-arch': [],
                 'binary': [
                         [rel({'name': 'binutils'})],
                         [rel({'name': 'binutils-dev'})],
@@ -1519,7 +1635,7 @@ class TestVersionAccessor(unittest.TestCase):
         v = Version(newver)
         p.set_version(v)
         self.assertEqual(p['Version'], newver)
-        self.assertTrue(isinstance(p['Version'], six.string_types))
+        self.assertTrue(isinstance(p['Version'], str))
 
 
 @unittest.skipUnless(os.path.exists('/usr/bin/gpgv'), "gpgv not installed")
@@ -1558,7 +1674,7 @@ class TestGpgInfo(unittest.TestCase):
 
     def test_from_sequence_newline_terminated(self):
         # type: () -> None
-        sequence = BytesIO(self.data)
+        sequence = io.BytesIO(self.data)
         gpg_info = deb822.GpgInfo.from_sequence(sequence, keyrings=[KEYRING])
         self._validate_gpg_info(gpg_info)
 
@@ -1583,138 +1699,5 @@ class TestGpgInfo(unittest.TestCase):
         self._validate_gpg_info(gpg_info)
 
 
-def _no_space(s):
-    # type: (str) -> str
-    """Returns s.  Raises ValueError if s contains any whitespace."""
-    if re.search(r'\s', s):
-        raise ValueError('whitespace not allowed')
-    return s
-
-
-class RestrictedWrapperTest(unittest.TestCase):
-    class Wrapper(deb822.RestrictedWrapper):
-        restricted_field = deb822.RestrictedField('Restricted-Field')
-        required_field = deb822.RestrictedField('Required-Field', allow_none=False)
-        space_separated = deb822.RestrictedField(
-                'Space-Separated',
-                from_str=lambda s: tuple((s or '').split()),
-                to_str=lambda seq: ' '.join(_no_space(s) for s in seq) or None)
-
-    def test_unrestricted_get_and_set(self):
-        # type: () -> None
-        data = deb822.Deb822()
-        data['Foo'] = 'bar'
-
-        wrapper = self.Wrapper(data)
-        self.assertEqual('bar', wrapper['Foo'])
-        wrapper['foo'] = 'baz'
-        self.assertEqual('baz', wrapper['Foo'])
-        self.assertEqual('baz', wrapper['foo'])
-
-        multiline = 'First line\n Another line'
-        wrapper['X-Foo-Bar'] = multiline
-        self.assertEqual(multiline, wrapper['X-Foo-Bar'])
-        self.assertEqual(multiline, wrapper['x-foo-bar'])
-
-        expected_data = deb822.Deb822()
-        expected_data['Foo'] = 'baz'
-        expected_data['X-Foo-Bar'] = multiline
-        self.assertEqual(expected_data.keys(), data.keys())
-        self.assertEqual(expected_data, data)
-
-    def test_trivially_restricted_get_and_set(self):   # type: ignore
-        # mypy can't cope with the metaprogramming here
-        data = deb822.Deb822()
-        data['Required-Field'] = 'some value'
-
-        wrapper = self.Wrapper(data)
-        self.assertEqual('some value', wrapper.required_field)
-        self.assertEqual('some value', wrapper['Required-Field'])
-        self.assertEqual('some value', wrapper['required-field'])
-        self.assertIsNone(wrapper.restricted_field)
-
-        with self.assertRaises(deb822.RestrictedFieldError):
-            wrapper['Required-Field'] = 'foo'
-        with self.assertRaises(deb822.RestrictedFieldError):
-            wrapper['required-field'] = 'foo'
-        with self.assertRaises(deb822.RestrictedFieldError):
-            wrapper['Restricted-Field'] = 'foo'
-        with self.assertRaises(deb822.RestrictedFieldError):
-            wrapper['Restricted-field'] = 'foo'
-
-        with self.assertRaises(deb822.RestrictedFieldError):
-            del wrapper['Required-Field']
-        with self.assertRaises(deb822.RestrictedFieldError):
-            del wrapper['required-field']
-        with self.assertRaises(deb822.RestrictedFieldError):
-            del wrapper['Restricted-Field']
-        with self.assertRaises(deb822.RestrictedFieldError):
-            del wrapper['restricted-field']
-
-        with self.assertRaises(TypeError):
-            wrapper.required_field = None           # type: ignore
-
-        wrapper.restricted_field = 'special value'  # type: ignore
-        self.assertEqual('special value', data['Restricted-Field'])
-        wrapper.restricted_field = None             # type: ignore
-        self.assertFalse('Restricted-Field' in data)
-        self.assertIsNone(wrapper.restricted_field)
-
-        wrapper.required_field = 'another value'    # type: ignore
-        self.assertEqual('another value', data['Required-Field'])
-
-    def test_set_already_none_to_none(self):   # type: ignore
-        # mypy can't cope with the metaprogramming here
-        data = deb822.Deb822()
-        wrapper = self.Wrapper(data)
-        wrapper.restricted_field = 'Foo'   # type: ignore
-        wrapper.restricted_field = None    # type: ignore
-        self.assertFalse('restricted-field' in data)
-        wrapper.restricted_field = None    # type: ignore
-        self.assertFalse('restricted-field' in data)
-
-    def test_processed_get_and_set(self):   # type: ignore
-        # mypy can't cope with the metaprogramming here
-        data = deb822.Deb822()
-        data['Space-Separated'] = 'foo bar baz'
-
-        wrapper = self.Wrapper(data)
-        self.assertEqual(('foo', 'bar', 'baz'), wrapper.space_separated)
-        wrapper.space_separated = ['bar', 'baz', 'quux']     # type: ignore
-        self.assertEqual('bar baz quux', data['space-separated'])
-        self.assertEqual('bar baz quux', wrapper['space-separated'])
-        self.assertEqual(('bar', 'baz', 'quux'), wrapper.space_separated)
-
-        with self.assertRaises(ValueError) as cm:
-            wrapper.space_separated = ('foo', 'bar baz')    # type: ignore
-        self.assertEqual(('whitespace not allowed',), cm.exception.args)
-
-        wrapper.space_separated = None     # type: ignore
-        self.assertEqual((), wrapper.space_separated)
-        self.assertFalse('space-separated' in data)
-        self.assertFalse('Space-Separated' in data)
-
-        wrapper.space_separated = ()     # type: ignore
-        self.assertEqual((), wrapper.space_separated)
-        self.assertFalse('space-separated' in data)
-        self.assertFalse('Space-Separated' in data)
-
-    def test_dump(self):   # type: ignore
-        # mypy can't cope with the metaprogramming here
-        data = deb822.Deb822()
-        data['Foo'] = 'bar'
-        data['Baz'] = 'baz'
-        data['Space-Separated'] = 'baz quux'
-        data['Required-Field'] = 'required value'
-        data['Restricted-Field'] = 'restricted value'
-
-        wrapper = self.Wrapper(data)
-        self.assertEqual(data.dump(), wrapper.dump())
-
-        wrapper.restricted_field = 'another value'        # type: ignore
-        wrapper.space_separated = ('bar', 'baz', 'quux')  # type: ignore
-        self.assertEqual(data.dump(), wrapper.dump())
-
-
 if __name__ == '__main__':
     unittest.main()
